/*************************************************************************************************

Variable Substituter Utility Class
Licensed to users of N2CMS under the terms of the Boost Software License

Copyright (c) 2013 Benjamin Herila <mailto:ben@herila.net>

Boost Software License - Version 1.0 - August 17th, 2003

Permission is hereby granted, free of charge, to any person or organization obtaining a copy of the 
software and accompanying documentation covered by this license (the "Software") to use, reproduce,
display, distribute, execute, and transmit the Software, and to prepare derivative works of the
Software, and to permit third-parties to whom the Software is furnished to do so, all subject to 
the following:

The copyright notices in the Software and this entire statement, including the above license grant,
this restriction and the following disclaimer, must be included in all copies of the Software, in
whole or in part, and all derivative works of the Software, unless such copies or derivative works 
are solely in the form of machine-executable object code generated by a source language processor.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-
INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*************************************************************************************************/

using System;
using System.Linq;
using System.Text.RegularExpressions;

namespace N2
{

    class VariableSubstituter
    {
        internal static Regex PropertyReplacementRegex = new Regex("(?:\\$\\$|{{)(.*?)(?:\\$\\$|}})"); // allow {{token}} or $$token$$

        internal static string Substitute(string p, ContentItem item)
        {
            return PropertyReplacementRegex.Replace(p, match => ComputeReplacement(item, match));
        }

        private static string ComputeReplacement(ContentItem item, Match match)
        {
            var pn = match.Groups[1].Value.Split(new[] {':','|'}, 2); // allow : or | as token parameter separators
            try
            {
                var d1 = item.GetDetail(pn[0]);
                if (d1 == null)
                {
                    // hard-coded a few properties here to avoid reflection
                    if (pn[0].Equals("Title", StringComparison.OrdinalIgnoreCase))
                        d1 = item.Title;
                    else if (pn[0].Equals("Url", StringComparison.OrdinalIgnoreCase))
                        d1 = item.Url;
                    else if (pn[0].Equals("Id", StringComparison.OrdinalIgnoreCase))
                        d1 = item.ID;
                    else if (pn[0].Equals("Published", StringComparison.OrdinalIgnoreCase))
                        d1 = item.Published;
                    else if (pn[0].Equals("TranslationKey", StringComparison.OrdinalIgnoreCase))
                        d1 = item.TranslationKey;
                    else if (pn[0].Equals("SavedBy", StringComparison.OrdinalIgnoreCase))
                        d1 = item.SavedBy;
                    else if (pn[0].Equals("Updated", StringComparison.OrdinalIgnoreCase))
                        d1 = item.Updated;
                    else if (pn[0].Equals("Published", StringComparison.OrdinalIgnoreCase))
                        d1 = item.Published;
                    else if (pn[0].Equals("Path", StringComparison.OrdinalIgnoreCase))
                        d1 = item.Path;
                    else
                    {
                        // Use Reflection to resolve property. 
                        var type = item.GetType();
                        var props = type.GetProperties().Where(f => f.Name.Equals(pn[0], StringComparison.OrdinalIgnoreCase)).ToArray();
                        if (props.Length > 0)
                            d1 = props[0].GetValue(item, null);
                            // it's a property
                        else
                        {
                            var fields = type.GetFields().Where(f => f.Name.Equals(pn[0], StringComparison.OrdinalIgnoreCase)).ToArray();
                            if (fields.Length > 0)
                                d1 = fields[0].GetValue(item); // it's a field
                        }
                    }
                }
                if (d1 == null)
                    return String.Concat('{', pn[0], ":null}");
                return (pn.Length == 2
                            ? String.Format(String.Concat("{0:", pn[1], '}'), d1)
                            : d1.ToString());
            }
            catch (Exception err)
            {
                return err.ToString();
            }
        }
    }
}
